@@ -8,7 +8,7 @@
  *                    on x86 processors
  *
  *      Version:   4.1
- *      Released:  13.6.2016
+ *      Released:  8.8.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -35,7 +35,7 @@ package.path = '<INSTALLED_PREFIX>/share/lua/?.lua;' .. package.path
 local likwid = require("likwid")
 
 print_stdout = print
-print_stderr = function(...) for k,v in pairs({...}) do io.stderr:write(v .. "\n") end end
+print_stderr = function(...) for k,v in pairs({...}) do io.stderr:write(v .. "\n") end io.stderr:flush() end
 
 local function version()
     print_stdout(string.format("likwid-perfctr --  Version %d.%d",likwid.version,likwid.release))
@@ -248,7 +248,7 @@ for opt,arg in likwid.getopt(arg, {"a", "c:", "C:", "e", "E:", "g:", "h", "H", "
         use_wrapper = true
     elseif (opt == "S") then
         use_stethoscope = true
-        if arg ~= nil and arg:match("%d%a?+s") then
+        if arg ~= nil and arg:match("%d+%a?s") then
             duration = likwid.parse_time(arg)
         else
             print_stderr("Option requires an argument")
@@ -366,7 +366,7 @@ if num_cpus > 0 then
         for j, cpu2 in pairs(cpulist) do
             if i ~= j and cpu1 == cpu2 then
                 print_stderr("List of CPUs is not unique, got two times CPU " .. tostring(cpu1))
-                if likwid.access(outfile..".tmp", "e") == 0 then
+                if outfile and likwid.access(outfile..".tmp", "e") == 0 then
                     os.remove(outfile..".tmp")
                 end
                 os.exit(1)
@@ -754,6 +754,10 @@ if use_wrapper or use_timeline then
 
     if not pid then
         print_stderr("Failed to execute command: ".. execString)
+        likwid.putTopology()
+        likwid.putNumaInfo()
+        likwid.putConfiguration()
+        os.exit(1)
     end
     start = likwid.startClock()
     groupTime[activeGroup] = 0
