@@ -8,7 +8,7 @@
  *      consumption on architectures implementing the RAPL interface.
  *
  *      Version:   4.1
- *      Released:  13.6.2016
+ *      Released:  8.8.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -346,7 +346,7 @@ if not print_info and not print_temp then
             print_stdout(string.format("Measure for socket %d on CPU %d", socket,cpu ))
             for j, dom in pairs(domainList) do
                 if power["domains"][dom]["supportStatus"] then
-                    local energy = likwid.calcPower(before[cpu][dom], after[cpu][dom], 0)
+                    local energy = likwid.calcPower(before[cpu][dom], after[cpu][dom], j-1)
                     print_stdout(string.format("Domain %s:", dom))
                     print_stdout(string.format("Energy consumed: %g Joules",energy))
                     print_stdout(string.format("Power consumed: %g Watt",energy/runtime))
