@@ -8,7 +8,7 @@
  *                    on x86 processors.
  *
  *      Version:   4.1
- *      Released:  13.6.2016
+ *      Released:  8.8.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -332,9 +332,9 @@ if print_graphical and not print_csv then
                     local tmpString = ""
                     local cacheWidth = 0
                     if cputopo["cacheLevels"][cache]["size"] < 1048576 then
-                        tmpString = string.format("%dkB", cputopo["cacheLevels"][cache]["size"]/1024)
+                        tmpString = string.format("%.0f kB", cputopo["cacheLevels"][cache]["size"]/1024)
                     else
-                        tmpString = string.format("%dMB", cputopo["cacheLevels"][cache]["size"]/1048576)
+                        tmpString = string.format("%.0f MB", cputopo["cacheLevels"][cache]["size"]/1048576)
                     end
                     if sharedCores > 1 then
                         if sharedCores > cputopo["numCoresPerSocket"] then
